@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009, 2010, 2011, 2013 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,6 +12,7 @@
 # General Public License for more details.
 
 import argparse
+import cgi
 import locale
 import os
 import re
@@ -43,12 +44,16 @@ class ArgumentParser(argparse.ArgumentParser):
         version = '%(prog)s ' + __version__
         argparse.ArgumentParser.__init__(self, usage=usage)
         self.add_argument('-v', '--version', action='version', version=version, help='show version information and exit')
-        self.add_argument('-p', '--pages', dest='pages', action='store', default=None, help='pages to convert')
-        self.add_argument('path', metavar='FILE', help='DjVu file to process')
+        group = self.add_argument_group(title='input selection options')
+        group.add_argument('path', metavar='FILE', help='DjVu file to covert')
+        group.add_argument('-p', '--pages', dest='pages', action='store', default=None, help='pages to convert')
         group = self.add_argument_group(title='word segmentation options')
-        group.add_argument('--word-segmentation', dest='word_segmentation', choices=('simple', 'uax29'), default='space', help='word segmentation algorithm')
+        group.add_argument('--word-segmentation', dest='word_segmentation', choices=('simple', 'uax29'), default='simple', help='word segmentation algorithm')
         # -l/--language is currently not very useful, as ICU don't have any specialisations for languages ocrodjvu supports:
         group.add_argument('-l', '--language', dest='language', help=argparse.SUPPRESS or 'language for word segmentation', default='eng')
+        group = self.add_argument_group(title='HTML output options')
+        group.add_argument('--title', dest='title', help='document title', default='DjVu hidden text layer')
+        group.add_argument('--css', metavar='STYLE', dest='css', help='CSS style', default='')
 
     def parse_args(self, args=None, namespace=None):
         options = argparse.ArgumentParser.parse_args(self, args, namespace)
@@ -264,18 +269,22 @@ def process_page(page_text, options):
     tree = etree.ElementTree(result)
     tree.write(sys.stdout, encoding='UTF-8')
 
-hocr_header = '''\
+hocr_header_template = '''\
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml">
 <head>
-  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
+  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
   <meta name="ocr-system" content="%(ocr_system)s" />
   <meta name="ocr-capabilities" content="%(ocr_capabilities)s" />
-  <title>DjVu hidden text layer</title>
+  <style type="text/css">%(css)s</style>
+  <title>%(title)s</title>
 </head>
 <body>
 '''
+
+hocr_header_style_re = re.compile('^\s+<style\s.*?\n', re.MULTILINE)
+
 hocr_footer = '''
 </body>
 </html>
@@ -303,11 +312,15 @@ def main(argv=sys.argv):
         ['djvused', '-f', sed_script.name, os.path.abspath(options.path)],
         stdout=ipc.PIPE,
     )
-    sys.stdout.write(
-        hocr_header % dict(
-            ocr_system='djvu2hocr %s' % __version__,
-            ocr_capabilities=' '.join(hocr.djvu2hocr_capabilities)
-    ))
+    hocr_header = hocr_header_template % dict(
+        ocr_system='djvu2hocr %s' % __version__,
+        ocr_capabilities=' '.join(hocr.djvu2hocr_capabilities),
+        title=cgi.escape(options.title),
+        css=cgi.escape(options.css),
+    )
+    if not options.css:
+        hocr_header = re.sub(hocr_header_style_re, '', hocr_header, count=1)
+    sys.stdout.write(hocr_header)
     for n in page_iterator:
         try:
             page_size = [
