@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2008, 2009, 2010, 2011, 2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008, 2009, 2010, 2011, 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ import inspect
 import locale
 import os.path
 import shutil
+import string
 import sys
 import threading
 import traceback
@@ -127,6 +128,37 @@ def get_cpu_count():
     except (ValueError, OSError, AttributeError):
         return 1
 
+def expand_template(template, pageno, pageid):
+    d = {
+        'page': pageno,
+        'id': pageid,
+        'id-ext': os.path.splitext(pageid)[0],
+    }
+    formatter = string.Formatter()
+    for _, var, _, _ in formatter.parse(template):
+        if var is None:
+            continue
+        if '+' in var:
+            sign = +1
+            base_var, offset = var.split('+')
+        elif '-' in var:
+            sign = -1
+            base_var, offset = var.split('-')
+        else:
+            continue
+        try:
+            offset = sign * int(offset, 10)
+        except ValueError:
+            continue
+        try:
+            base_value = d[base_var]
+        except LookupError:
+            continue
+        if not isinstance(base_value, int):
+            continue
+        d[var] = d[base_var] + offset
+    return formatter.vformat(template, (), d)
+
 class ArgumentParser(argparse.ArgumentParser):
 
     savers = BundledSaver, IndirectSaver, ScriptSaver, InPlaceSaver, DryRunSaver
@@ -150,7 +182,7 @@ class ArgumentParser(argparse.ArgumentParser):
         version = '%(prog)s ' + __version__
         argparse.ArgumentParser.__init__(self, usage=usage)
         self.add_argument('-v', '--version', action='version', version=version, help='show version information and exit')
-        saver_group = self.add_argument_group(title='options controlling output')
+        group = self.add_argument_group(title='options controlling output')
         for saver_type in self.savers:
             options = saver_type.options
             try:
@@ -161,7 +193,7 @@ class ArgumentParser(argparse.ArgumentParser):
             metavar = None
             if n_args == 1:
                 metavar = 'FILE'
-            saver_group.add_argument(
+            group.add_argument(
                 *options,
                 **dict(
                     metavar=metavar,
@@ -170,10 +202,12 @@ class ArgumentParser(argparse.ArgumentParser):
                     help=saver_type.__doc__
                 )
             )
+        group.add_argument('--ocr-only', dest='ocr_only', action='store_true', default=False, help='''don't save pages without OCR''')
+        group.add_argument('--clear-text', dest='clear_text', action='store_true', default=False, help='remove existing hidden text')
+        group.add_argument('--save-raw-ocr', dest='save_raw_ocr_dir', metavar='DIRECTORY', help='save raw OCR output')
+        group.add_argument('--raw-ocr-filename-template', metavar='TEMPLATE', default='{id-ext}', help='file naming scheme for raw OCR')
         self.add_argument('-e', '--engine', dest='engine', nargs=1, action=self.set_engine, default=self.default_engine, help='OCR engine to use')
         self.add_argument('--list-engines', action=self.list_engines, nargs=0, help='print list of available OCR engines')
-        self.add_argument('--ocr-only', dest='ocr_only', action='store_true', default=False, help='''don't save pages without OCR''')
-        self.add_argument('--clear-text', dest='clear_text', action='store_true', default=False, help='remove existing hidden text')
         self.add_argument('-l', '--language', dest='language', help='set recognition language')
         self.add_argument('--list-languages', action=self.list_languages, nargs=0, help='print list of available languages')
         self.add_argument('--render', dest='render_layers', choices=self._render_map.keys(), action='store', default='mask', help='image layers to render')
@@ -214,7 +248,7 @@ class ArgumentParser(argparse.ArgumentParser):
     class list_languages(argparse.Action):
         def __call__(self, parser, namespace, values, option_string=None):
             try:
-                for language in namespace.engine().list_languages():
+                for language in sorted(namespace.engine().list_languages()):
                     print language
             except errors.EngineNotFound, ex:
                 logger.error(ex)
@@ -255,6 +289,17 @@ class ArgumentParser(argparse.ArgumentParser):
                 'You must use exactly one of the following options: %s' %
                 ', '.join('/'.join(saver.options) for saver in self.savers)
             )
+        if options.save_raw_ocr_dir is not None:
+            try:
+                os.stat(os.path.join(options.save_raw_ocr_dir, ''))
+            except EnvironmentError, ex:
+                self.error('cannot open %r: %s' % (ex.filename, ex[1]))
+            try:
+                expand_template(options.raw_ocr_filename_template, pageno=0, pageid='')
+            except ValueError, ex:
+                self.error('cannot parse filename template %r: %s' % (options.raw_ocr_filename_template, ex))
+            except KeyError, ex:
+                self.error('cannot parse filename template %r: unknown field %r' % (options.raw_ocr_filename_template, ex.args[0]))
         # It might be tempting to verify language name correctness at argument
         # parse time (rather than after argument parsing). However, it's
         # desirable to be able to specify a language *before* specifying an OCR
@@ -276,8 +321,9 @@ class ArgumentParser(argparse.ArgumentParser):
         except errors.EngineNotFound, ex:
             self.error(str(ex))
         try:
-            if not options.engine.has_language(options.language):
-                self.error('Language pack for the selected language (%s) is not available.' % options.language)
+            options.engine.check_language(options.language)
+        except errors.MissingLanguagePack, ex:
+            self.error(str(ex))
         except errors.InvalidLanguageId, ex:
             self.error(str(ex))
         except errors.UnknownLanguageList:
@@ -320,35 +366,39 @@ class Context(djvu.decode.Context):
         finally:
             file.close()
 
+    def save_raw_ocr(self, page, result):
+        output_dir = self._options.save_raw_ocr_dir
+        if output_dir is None:
+            return
+        template = self._options.raw_ocr_filename_template
+        pageid = page.file.id
+        pageno = page.n + 1
+        prefix = os.path.join(
+            output_dir,
+            expand_template(template, pageno=pageno, pageid=pageid),
+        )
+        result.save(prefix)
+
     def process_page(self, page):
         logger.info('- Page #%d', page.n + 1)
         page_job = page.decode(wait=True)
         size = page_job.size
         with self.get_output_image(page.n, page_job) as pfile:
-            result_file = None
-            with self._engine.recognize(pfile, language=self._options.language, details=self._options.details, uax29=self._options.uax29) as result:
-                try:
-                    if self._debug:
-                        result_file = self._temp_file('%06d.%s' % (page.n, self._engine.output_format))
-                        result_data = result.read()
-                        result_file.write(result_data)
-                        result_file.seek(0)
-                    else:
-                        result_file = result
-                    [text] = self._engine.extract_text(result_file,
-                        rotation=page.rotation,
-                        details=self._options.details,
-                        uax29=self._options.uax29,
-                        html5=self._options.html5,
-                        fix_utf8=self._engine.needs_utf8_fix,
-                        page_size=size
-                    )
-                    # It should be: (page 0 0 <width> <height> …):
-                    assert len(text) > 5
-                    return text
-                finally:
-                    if result_file is not None:
-                        result_file.close()
+            result = self._engine.recognize(pfile, language=self._options.language, details=self._options.details, uax29=self._options.uax29)
+            if self._debug:
+                result.save(os.path.join(self._temp_dir, '%06d' % page.n))
+            self.save_raw_ocr(page, result)
+            [text] = self._engine.extract_text(result.as_stringio(),
+                rotation=page.rotation,
+                details=self._options.details,
+                uax29=self._options.uax29,
+                html5=self._options.html5,
+                fix_utf8=self._engine.needs_utf8_fix,
+                page_size=size
+            )
+            # It should be: (page 0 0 <width> <height> …):
+            assert len(text) > 5
+            return text
 
     def page_thread(self, pages, results, condition):
         for page in pages:
