@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010, 2011, 2013 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,6 @@
 
 from __future__ import division
 
-import contextlib
 import functools
 import re
 import shlex
@@ -34,6 +33,7 @@ from .. import utils
 const = text_zones.const
 
 _default_language = 'eng'
+_language_pattern = re.compile('^[a-z]{3}$')
 _version_re = re.compile(r'\bgocr ([0-9]+).([0-9]+)\b')
 
 class ExtractSettings(object):
@@ -108,7 +108,6 @@ class Engine(common.Engine):
 
     name = 'gocr'
     image_format = image_io.PNM
-    output_format = 'gocr.xml'
 
     executable = utils.property('gocr')
     extra_args = utils.property([], shlex.split)
@@ -143,20 +142,25 @@ class Engine(common.Engine):
     def get_default_language(cls):
         return _default_language
 
-    def has_language(self, language):
-        return language == _default_language
+    def check_language(self, language):
+        if not _language_pattern.match(language):
+            raise errors.InvalidLanguageId(language)
+        if language != _default_language:
+            raise errors.MissingLanguagePack(language)
 
     def list_languages(self):
         yield _default_language
 
-    @contextlib.contextmanager
     def recognize(self, image, language, details=None, uax29=None):
         worker = ipc.Subprocess(
             [self.executable, '-i', image.name, '-f', 'XML'] + self.extra_args,
             stdout=ipc.PIPE,
         )
         try:
-            yield worker.stdout
+            return common.Output(
+                worker.stdout.read(),
+                format='gocr.xml',
+            )
         finally:
             worker.wait()
 
