@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010, 2011, 2013 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,7 +11,6 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
-import contextlib
 import functools
 import re
 import shlex
@@ -102,7 +101,6 @@ class Engine(common.Engine):
 
     name = 'ocrad'
     image_format = image_io.PNM
-    output_format = 'orf'
 
     executable = utils.property('ocrad')
     extra_args = utils.property([], shlex.split)
@@ -139,10 +137,11 @@ class Engine(common.Engine):
                 raise errors.UnknownLanguageList
         return result
 
-    def has_language(self, language):
+    def check_language(self, language):
         if not _language_pattern.match(language):
             raise errors.InvalidLanguageId(language)
-        return language in self._languages
+        if language not in self._languages:
+            raise errors.MissingLanguagePack(language)
 
     @classmethod
     def get_default_language(cls):
@@ -151,7 +150,6 @@ class Engine(common.Engine):
     def list_languages(self):
         return iter(self._languages)
 
-    @contextlib.contextmanager
     def recognize(self, image, language, details=None, uax29=None):
         charset = 'iso-8859-15'
         if language == 'tur':
@@ -161,7 +159,10 @@ class Engine(common.Engine):
             stdout=ipc.PIPE,
         )
         try:
-            yield worker.stdout
+            return common.Output(
+                worker.stdout.read(),
+                format='orf',
+            )
         finally:
             worker.wait()
 
