@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2008, 2009, 2010, 2011, 2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008, 2009, 2010, 2011, 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -88,17 +88,21 @@ bboxes_re = re.compile(
         (?: ,? \s* (?: -?\d+ \s+ -?\d+ \s+ -?\d+ \s+ -?\d+) )* )
     ''', re.VERBOSE)
 
+tesseract_rstrip = functools.partial(
+    re.compile('\n\s+$').sub,
+    ''
+)
+
 def _apply_bboxes(djvu_class, bbox_source, text, settings, page_size):
     embedded_eol = False
     if djvu_class <= const.TEXT_ZONE_LINE:
         if text.endswith('\n'):
             embedded_eol = True
     if settings.tesseract:
-        # Tesseract ≥ 3.00 uses space for characters it couldn't recognize
-        # (yay, excellent choice), so let's treat them just like ordinary
-        # characters. However, trailing newline characters can appear at the
-        # end of line.
-        new_text = text.rstrip('\n')
+        # Tesseract ≥ 3.00 uses space for characters it couldn't recognize, so
+        # let's treat them just like ordinary characters. However, trailing
+        # newline characters can appear at the end of line.
+        new_text = tesseract_rstrip(text)
     else:
         # Cuneiform tends to attach superfluous whitespace.
         # Also, a newline character can appear at the end of line.
@@ -199,6 +203,17 @@ def _scan(node, settings, page_size=None):
     if not isinstance(node.tag, basestring) or node.tag == 'script':
         # Ignore non-elements.
         return []
+
+    title = node.get('title') or ''
+    m = bbox_re.search(title)
+    if m is None:
+        bbox = text_zones.BBox()
+    else:
+        bbox = text_zones.BBox(
+            *(int(m.group(ident))
+            for ident in ('x0', 'y0', 'x1', 'y1'))
+        )
+
     if settings.cuneiform and settings.cuneiform <= (0, 8):
         # Cuneiform ≤ 0.8 don't mark OCR elements in an hOCR way.
         djvu_class = cuneiform_tag_to_djvu(node.tag)
@@ -206,8 +221,9 @@ def _scan(node, settings, page_size=None):
         hocr_classes = (node.get('class') or '').split()
         djvu_class = None
         for hocr_class in hocr_classes:
-            if settings.tesseract and hocr_class == 'ocrx_word':
-                # Tesseract > 3.00 use ocrx_word for its own purposes.
+            if settings.tesseract and hocr_class == 'ocrx_word' and not bbox:
+                # Some versions of Tesseract > 3.00 use ocrx_word for its own
+                # purposes.
                 pass
             else:
                 djvu_class = hocr_class_to_djvu(hocr_class)
@@ -221,15 +237,6 @@ def _scan(node, settings, page_size=None):
     if not djvu_class:
         # Just process our children.
         return get_children(node)
-    title = node.get('title') or ''
-    m = bbox_re.search(title)
-    if m is None:
-        bbox = text_zones.BBox()
-    else:
-        bbox = text_zones.BBox(
-            *(int(m.group(ident))
-            for ident in ('x0', 'y0', 'x1', 'y1'))
-        )
 
     if djvu_class is const.TEXT_ZONE_PAGE:
         if not bbox:
@@ -283,8 +290,9 @@ def _scan(node, settings, page_size=None):
             for child in children:
                 if isinstance(child, text_zones.Zone):
                     bbox.update(child.bbox)
-        if len(children) == 0:
-            return []
+        if djvu_class >= const.TEXT_ZONE_LINE:
+            if isinstance(children[-1], basestring) and children[-1].isspace():
+                del children[-1]
 
     if djvu_class <= const.TEXT_ZONE_WORD:
         if has_zone:
@@ -446,19 +454,20 @@ def extract_text(stream, **kwargs):
     '''
     settings = ExtractSettings(**kwargs)
     doc = read_document(stream, settings)
-    if doc.find('/head/meta[@name="ocr-capabilities"]') is None:
-        ocr_system = doc.find('/head/meta[@name="ocr-system"]')
-        if ocr_system is None:
+    ocr_system = doc.find('/head/meta[@name="ocr-system"]')
+    if ocr_system is None:
+        if doc.find('/head/meta[@name="ocr-capabilities"]') is None:
             # This is wild guess. However, since ocr-system is a required meta
             # tag, the hOCR we are processing is broken anyway.
             settings.cuneiform = (0, 8)
-        elif ocr_system.get('content') == 'openocr':
-            settings.cuneiform = (0, 9)
-        elif ocr_system.get('content') == 'tesseract':
-            settings.tesseract = True
+    elif ocr_system.get('content') == 'openocr':
+        settings.cuneiform = (0, 9)
+    elif ocr_system.get('content').split()[0] == 'tesseract':
+        settings.tesseract = True
     if settings.details < TEXT_DETAILS_WORD or (settings.uax29 and settings.details <= text_zones.TEXT_DETAILS_WORD):
-        tesseract_bbox_data = doc.find('//script[@type="ocrodjvu/tesseract"]')
+        tesseract_bbox_data = doc.find('//script[@type="application/x-ocrodjvu-tesseract"]')
         if tesseract_bbox_data is not None:
+            settings.tesseract = True
             tesseract_bbox_data = extract_tesseract_bbox_data(tesseract_bbox_data)
             settings.bbox_data = tesseract_bbox_data
     scan_result = scan(doc.find('/body'), settings)
